/**********************************************************************
* HEADER_START                                                        *
*                                                                     *
*      Name:           HC08SPRG.H                                     *
*      Project:        Developper's HC08 Bootloader Master            *
*      Description:    Main header file                               *
*      Platform:       Linux/Win32                                    *
*      Revision:       1.0                                            *
*      Date:           Apr 8 2002                                     *
*      Author:         Michal Hanak                                   *
*      Company:        Motorola SPS                                   *
*      Security:       General Business                               *
*                                                                     *
* =================================================================== *
* Copyright (c):      MOTOROLA Inc., 2001, All rights reserved.       *
*                                                                     *
* =================================================================== *
* THIS SOFTWARE IS PROVIDED BY MOTOROLA "AS IS" AND ANY               *
* EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR  *
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL MOTOROLA OR              *
* ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    *
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT        *
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        *
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)            *
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, *
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       *
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED *
* OF THE POSSIBILITY OF SUCH DAMAGE.                                  *
* =================================================================== *
*
* HEADER_END
*/

#ifndef _HC08SPRG_H
#define _HC08SPRG_H

// types

typedef struct
{
	unsigned mem_start;		// start of usable flash
	unsigned mem_end;		// end of usable flash
	unsigned erblk;			// erase block size
	unsigned wrblk;			// write block size
	unsigned bl_tbl;		// start of boot-loader table (private data and jump table)
	unsigned int_vect_tbl;	// start of hard-wired interrupt vectors
	BYTE priv_data[8];		// 8 bytes of private info
	char targ_name[40];		// target name
} IDENT_DATA;

typedef struct {
	BYTE d[0x10000];	// data
	BYTE f[0x10000];	// valid flag 0=empty; 1=usercode; 2=systemcode
} BOARD_MEM;

// globals

extern IDENT_DATA ident;
extern BOARD_MEM  image;
extern int bl_version;

// shared functions

int read_s19(char *fn);
int hook_reset(void);
int calibrate_speed(void);
int read_mcu_info(void);
int setup_vect_tbl(void);
int	prg_mem(void);
int	check_image(void);
int	unhook(void);

#endif
