/*
 * BaseControllerReader.java
 *
 * Created on May 14, 2003, 7:08 PM
 */

package org.megaJoltLiteJr;

/**
 *
 * @author  brent
 */
import javax.comm.*;
import java.io.*;
import java.util.*;
import java.util.TooManyListenersException;

public abstract class BaseControllerCommunicator implements SerialPortEventListener{
    
    SerialPort serialPort;
    
    /** Holds value of property serialInputStream. */
    private InputStream serialInputStream;
    
    /** Holds value of property serialOutputStream. */
    private OutputStream serialOutputStream;
    
    /** Holds value of property inputBuffer. */
    private List inputBuffer = new ArrayList();
    
    /** Holds value of property readComplete. */
    private boolean readComplete;
    
    /** Creates a new instance of BaseControllerReader */
    public BaseControllerCommunicator(SerialPort serialPort) {
        setSerialPort(serialPort);
    }

    public SerialPort getSerialPort(){
        return this.serialPort;
    }
    
    public void setSerialPort(SerialPort newSerialPort){
    
        this.serialPort=newSerialPort;
        
        try{
            this.setSerialInputStream(serialPort.getInputStream());
            this.setSerialOutputStream(serialPort.getOutputStream());
        }
        catch (IOException e){
            throw new CommRuntimeException("Unable to get input/output stream from port");
        }

        try {
            serialPort.removeEventListener();
            serialPort.addEventListener(this);
            newSerialPort.notifyOnDataAvailable(true);
        } catch (TooManyListenersException e) {throw new CommRuntimeException("Too Many event listeners registered for port");}
    
    }
    
    public void releaseSerialPort(){
     
        SerialPort serialPort=this.getSerialPort();
        
        if (null!=serialPort){
            serialPort.removeEventListener();
        }
    }

    

    /** Getter for property serialInputStream.
     * @return Value of property serialInputStream.
     *
     */
    public InputStream getSerialInputStream() {
        return this.serialInputStream;
    }
    
    /** Setter for property serialInputStream.
     * @param serialInputStream New value of property serialInputStream.
     *
     */
    public void setSerialInputStream(InputStream serialInputStream) {
        this.serialInputStream = serialInputStream;
    }
    
    /** Getter for property serialOutputStream.
     * @return Value of property serialOutputStream.
     *
     */
    public OutputStream getSerialOutputStream() {
        return this.serialOutputStream;
    }
    
    /** Setter for property serialOutputStream.
     * @param serialOutputStream New value of property serialOutputStream.
     *
     */
    public void setSerialOutputStream(OutputStream serialOutputStream) {
        this.serialOutputStream = serialOutputStream;
    }
    
    public final void serialEvent(SerialPortEvent event) {
        
        List inBuffer = this.getInputBuffer();
        
        switch(event.getEventType()) {
            case SerialPortEvent.BI:
            case SerialPortEvent.OE:
            case SerialPortEvent.FE:
            case SerialPortEvent.PE:
            case SerialPortEvent.CD:
            case SerialPortEvent.CTS:
            case SerialPortEvent.DSR:
            case SerialPortEvent.RI:
            case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
                break;
            case SerialPortEvent.DATA_AVAILABLE:
                byte[] readBuffer = new byte[1024];
                int numBytes=0;
                try {
                    InputStream inputStream = getSerialInputStream();
                    while (inputStream.available() > 0) {
                        numBytes = inputStream.read(readBuffer);
                    }
                    for (int i=0;i<numBytes;i++){
                        inBuffer.add(new Byte(readBuffer[i]));
                    }
                } catch (IOException e) {}
                break;
        }
        
        dataReceived(inBuffer);
        
    }
    
    /** Getter for property inputBuffer.
     * @return Value of property inputBuffer.
     *
     */
    public List getInputBuffer() {
        return this.inputBuffer;
    }
    
    /** Setter for property inputBuffer.
     * @param inputBuffer New value of property inputBuffer.
     *
     */
    public void setInputBuffer(List inputBuffer) {
        this.inputBuffer = inputBuffer;
    }
    
    public abstract void dataReceived(List inBuffer);
    
    /** Getter for property readComplete.
     * @return Value of property readComplete.
     *
     */
    public boolean isReadComplete() {
        return this.readComplete;
    }
    
    /** Setter for property readComplete.
     * @param readComplete New value of property readComplete.
     *
     */
    public void setReadComplete(boolean readComplete) {
        this.readComplete = readComplete;
    }
    
    protected int getUnsignedByteValue(List list, int index){
        
        return getUnsignedByteValue(((Byte)list.get(index)).byteValue());
    }
    
    protected int getUnsignedByteValue(byte b){
        
        int val = b < 0 ? ( Byte.MAX_VALUE + 1 ) * 2 + b : b;        
        return val;
        
    }
    

}
