/*
 * IgnitionConfig.java
 *
 * Created on May 19, 2003, 11:44 PM
 */

package org.megaJoltLiteJr;

/**
 *
 * @author  brent
 */
import java.util.*;

public class IgnitionConfig {

    public static final int RPM_BIN_SIZE=10;
    public static final int MAP_BIN_SIZE=10;
    public static final int CYLINDERS_4=4;
    public static final int CYLINDERS_6=6;
    public static final int CYLINDERS_8=8;
    public static final int MAX_RPM_BIN_VALUE=100;
    public static final int MIN_RPM_BIN_VALUE=5;
    public static final int MAX_MAP_BIN_VALUE=255;
    public static final int MIN_MAP_BIN_VALUE=0;
    public static final int MAX_IGNITION_ADVANCE_VALUE=59;
    public static final int USER_OUTPUT_SIZE=4;
    public static final int USER_OUTPUT_TYPE_MAP=0;
    public static final int USER_OUTPUT_TYPE_RPM=1;
    public static final int USER_OUTPUT_MODE_HIGH=0;
    public static final int USER_OUTPUT_MODE_LOW=1;
    
    private int[] rpmBins = new int[RPM_BIN_SIZE];
    private int[] mapBins = new int[MAP_BIN_SIZE];
    private int[][] ignitionMap = new int[RPM_BIN_SIZE][MAP_BIN_SIZE];
    private int cylinderCount;    
    private int[] userOutputValue = new int[USER_OUTPUT_SIZE];
    private int[] userOutputMode=new int[USER_OUTPUT_SIZE];
    private int[] userOutputType= new int[USER_OUTPUT_SIZE];
    private int shiftLightRpm;
    private int revLimitRpm;
    
    
    public int getUserOutputValue(int index){
        if (index<0 || index >=USER_OUTPUT_SIZE) throw new IllegalArgumentException("index specified out of range; must be >=0 and <=" + USER_OUTPUT_SIZE);
        return userOutputValue[index];
    }
    
    public void setUserOutputValue(int index, int value){
        if (index<0 || index >=USER_OUTPUT_SIZE) throw new IllegalArgumentException("index specified out of range; must be >=0 and <=" + USER_OUTPUT_SIZE);
        
        if (getUserOutputType(index)==USER_OUTPUT_TYPE_MAP){
            if (value<0 || value >MAX_MAP_BIN_VALUE) throw new IllegalArgumentException("MAP Value specified out of range; must be >=0 and <="+MAX_MAP_BIN_VALUE);
        }
        else if (getUserOutputType(index)==USER_OUTPUT_TYPE_RPM){
            if (value<0 || value >=MAX_RPM_BIN_VALUE) throw new IllegalArgumentException("RPM Value specified out of range; must be >=0 and <=" + MAX_RPM_BIN_VALUE);            
        }
        this.userOutputValue[index]=value;
    }
    
    public int getUserOutputType(int index){
        if (index<0 || index >= USER_OUTPUT_SIZE) throw new IllegalArgumentException("index specified out of range; must be >=0 and <=" + USER_OUTPUT_SIZE);
        return this.userOutputType[index];
    }
    
    public void setUserOutputType(int index, int type){
        if (index<0 || index >= USER_OUTPUT_SIZE) throw new IllegalArgumentException("index specified out of range; must be >=0 and <=" + USER_OUTPUT_SIZE);        
        if (type!=USER_OUTPUT_TYPE_MAP && type!=USER_OUTPUT_TYPE_RPM) throw new IllegalArgumentException("invalid user output type specified: " + type);
        this.userOutputType[index]=type;
    }
    
    public int getUserOutputMode(int index){
        if (index<0 || index >= USER_OUTPUT_SIZE) throw new IllegalArgumentException("index specified out of range; must be >=0 and <=" + USER_OUTPUT_SIZE);
        return this.userOutputMode[index];
        
    }
    
    public void setUserOutputMode(int index, int mode){
        if (index<0 || index >= USER_OUTPUT_SIZE) throw new IllegalArgumentException("index specified out of range; must be >=0 and <=" + USER_OUTPUT_SIZE);        
        if (mode!=USER_OUTPUT_MODE_HIGH && mode !=USER_OUTPUT_MODE_LOW) throw new IllegalArgumentException("invalid user output mode specified: " + mode);
        this.userOutputMode[index]=mode;
        
    }
    
    public void setIgnitionMapValue(int rpmBin,int mapBin, int value){
        
        if (value<0 || value > MAX_IGNITION_ADVANCE_VALUE) throw new IllegalArgumentException("value specified out of range: must be >=0 and <=" + MAX_IGNITION_ADVANCE_VALUE);
        if (rpmBin<0 || rpmBin >=RPM_BIN_SIZE) throw new IllegalArgumentException("RPM bin specified out of range: must be >=0 and <=" + (RPM_BIN_SIZE-1));
        if (mapBin<0 || mapBin >=MAP_BIN_SIZE) throw new IllegalArgumentException("MAP bin specified out of range: must be >=0 and <=" + (MAP_BIN_SIZE-1));
        
        ignitionMap[rpmBin][mapBin]=value;
    }
    
    public int getIgnitionMapValue(int rpmBin,int mapBin){
    
        return ignitionMap[rpmBin][mapBin];
    }
    
    public void setRpmBinValue(int rpmBin, int rpmValue){
        
        if (rpmBin<0 || rpmBin >=RPM_BIN_SIZE) throw new IllegalArgumentException("RPM bin specified out of range: must be >=0 and <=" + (RPM_BIN_SIZE-1));
        if (rpmValue<0 || rpmValue >=MAX_RPM_BIN_VALUE) throw new IllegalArgumentException("RPM Value specified out of range; must be >=0 and <=" + MAX_RPM_BIN_VALUE + "; was " + rpmValue);
        
        rpmBins[rpmBin]=rpmValue;
    }
    
    public int getRpmBinValue(int rpmBin){
        return rpmBins[rpmBin];
    }

    public void setMapBinValue(int mapBin, int mapValue){
        
        if (mapBin<0 || mapBin >=MAP_BIN_SIZE) throw new IllegalArgumentException("MAP bin specified out of range: must be >=0 and <=" + (MAP_BIN_SIZE-1));
        if (mapValue<MIN_MAP_BIN_VALUE || mapValue >=MAX_MAP_BIN_VALUE) throw new IllegalArgumentException("MAP Value specified out of range; must be >=0 and <="+MAX_MAP_BIN_VALUE);
        
        mapBins[mapBin]=mapValue;
    }
    
    public int getMapBinValue(int mapBin){
        return mapBins[mapBin];
    }
    
    /** Getter for property cylinderCount.
     * @return Value of property cylinderCount.
     *
     */
    public int getCylinderCount() {
        return this.cylinderCount;
    }
    
    /** Setter for property cylinderCount.
     * @param cylinderCount New value of property cylinderCount.
     *
     */
    public void setCylinderCount(int cylinderCount){

        if (cylinderCount!=CYLINDERS_4 && cylinderCount!=CYLINDERS_6 && cylinderCount!=CYLINDERS_8){
            throw new IllegalArgumentException("Invalid Cylinder Count specified: " + cylinderCount);
        }
        this.cylinderCount = cylinderCount;
    }
    
    /** Getter for property shiftLightRpm.
     * @return Value of property shiftLightRpm.
     *
     */
    public int getShiftLightRpm() {
        return this.shiftLightRpm;
    }
    
    /** Setter for property shiftLightRpm.
     * @param shiftLightRpm New value of property shiftLightRpm.
     *
     */
    public void setShiftLightRpm(int rpmValue) {

        if (rpmValue<0 || rpmValue >=MAX_RPM_BIN_VALUE) throw new IllegalArgumentException("RPM Value specified out of range; must be >=0 and <=" + MAX_RPM_BIN_VALUE + "; was " + rpmValue);
        this.shiftLightRpm = rpmValue;
    }
    
    /** Getter for property revLimitRpm.
     * @return Value of property revLimitRpm.
     *
     */
    public int getRevLimitRpm() {
        return this.revLimitRpm;
    }
    
    /** Setter for property revLimitRpm.
     * @param revLimitRpm New value of property revLimitRpm.
     *
     */
    public void setRevLimitRpm(int rpmValue) {

        if (rpmValue<0 || rpmValue >=MAX_RPM_BIN_VALUE) throw new IllegalArgumentException("RPM Value specified out of range; must be >=0 and <=" + MAX_RPM_BIN_VALUE + "; was " + rpmValue);
        this.revLimitRpm = rpmValue;
    }
    
}
