/*
 * IgnitionConfigReader.java
 *
 * Created on May 19, 2003, 11:59 PM
 */

package org.megaJoltLiteJr;

/**
 *
 * @author  brent
 */
import javax.comm.*;
import java.util.*;

import java.io.IOException;

public class IgnitionConfigReader extends BaseControllerCommunicator{

    public static final int RPM_BIN_OFFSET=0;
    public static final int MAP_BIN_OFFSET=10;
    public static final int IGNITION_MAP_OFFSET=20;
    public static final int CYLINDER_COUNT_OFFSET=120;
    public static final int USER_OUTPUT_CONFIG_OFFSET=121;
    public static final int USER_OUTPUT_VALUE_OFFSET=122;
    public static final int REV_LIMIT_OFFSET=126;
    public static final int SHIFT_LIGHT_OFFSET=127;
    
    
    public static final char READ_CONFIG_COMMAND='C';
    public static final long TIMEOUT=2000;
    public static final int READ_DATA_LENGTH=128;
    
    /** Creates a new instance of IgnitionConfigReader */
    public IgnitionConfigReader(SerialPort serialPort){
        super(serialPort);
    }
    
    public IgnitionConfig readIgnitionConfig(){
        
        long timeoutAt= System.currentTimeMillis()+IgnitionStateReader.TIMEOUT;
        
        this.getInputBuffer().clear();
        
        try{
            super.getSerialOutputStream().write(67);
        }
        catch (IOException e){
            throw new CommRuntimeException("Unable to send read Request",e);
        }
        
        while ( System.currentTimeMillis() < timeoutAt && !isReadComplete()){
            try{
                
                Thread.sleep(10);
            }
            catch (InterruptedException e){}
        }
        
        IgnitionConfig currentConfig = createConfig();
        
        return currentConfig;
    }
    
    public boolean isReadComplete(){

        
        List buffer = super.getInputBuffer();
        
        boolean readIsComplete = (buffer.size()>=READ_DATA_LENGTH);

        return readIsComplete;
    }
    
    private IgnitionConfig createConfig(){

        List readData = super.getInputBuffer();
        
        if (readData.size()==0) throw new NoControllerConnectionException("Could not connect to controller");
        if (readData.size()<READ_DATA_LENGTH) throw new IllegalStateException("Read buffer length not correct: was " + readData.size() + "; expected " + READ_DATA_LENGTH);
        
        IgnitionConfig config = new IgnitionConfig();
        
        for (int index=0;index<IgnitionConfig.RPM_BIN_SIZE;index++){
            int rpmBin = getUnsignedByteValue(readData,index+RPM_BIN_OFFSET);
            config.setRpmBinValue(index, rpmBin);
        }

        for (int index=0;index<IgnitionConfig.MAP_BIN_SIZE;index++){
            config.setMapBinValue(index, getUnsignedByteValue(readData, index+MAP_BIN_OFFSET));
        }
        
        for (int mapIndex=0;mapIndex<IgnitionConfig.MAP_BIN_SIZE;mapIndex++){
            for (int rpmIndex=0;rpmIndex<IgnitionConfig.RPM_BIN_SIZE;rpmIndex++){
                config.setIgnitionMapValue(rpmIndex, mapIndex, getUnsignedByteValue(readData, IGNITION_MAP_OFFSET+(mapIndex*IgnitionConfig.RPM_BIN_SIZE)+rpmIndex));
            }
        }
        
        config.setCylinderCount(getUnsignedByteValue(readData, CYLINDER_COUNT_OFFSET));

        int outputConfig=getUnsignedByteValue(readData, USER_OUTPUT_CONFIG_OFFSET);
        
        config.setUserOutputType(0, ((outputConfig & 1) == 0)?IgnitionConfig.USER_OUTPUT_TYPE_MAP:IgnitionConfig.USER_OUTPUT_TYPE_RPM);
        config.setUserOutputType(1, ((outputConfig & 2) == 0)?IgnitionConfig.USER_OUTPUT_TYPE_MAP:IgnitionConfig.USER_OUTPUT_TYPE_RPM);
        config.setUserOutputType(2, ((outputConfig & 4) == 0)?IgnitionConfig.USER_OUTPUT_TYPE_MAP:IgnitionConfig.USER_OUTPUT_TYPE_RPM);
        config.setUserOutputType(3, ((outputConfig & 8) == 0)?IgnitionConfig.USER_OUTPUT_TYPE_MAP:IgnitionConfig.USER_OUTPUT_TYPE_RPM);        

        config.setUserOutputMode(0, ((outputConfig & 16) == 0)?IgnitionConfig.USER_OUTPUT_MODE_LOW:IgnitionConfig.USER_OUTPUT_MODE_HIGH);
        config.setUserOutputMode(1, ((outputConfig & 32) == 0)?IgnitionConfig.USER_OUTPUT_MODE_LOW:IgnitionConfig.USER_OUTPUT_MODE_HIGH);
        config.setUserOutputMode(2, ((outputConfig & 64) == 0)?IgnitionConfig.USER_OUTPUT_MODE_LOW:IgnitionConfig.USER_OUTPUT_MODE_HIGH);
        config.setUserOutputMode(3, ((outputConfig & 128) == 0)?IgnitionConfig.USER_OUTPUT_MODE_LOW:IgnitionConfig.USER_OUTPUT_MODE_HIGH);
        
        
        for (int index=0;index<IgnitionConfig.USER_OUTPUT_SIZE;index++){
            config.setUserOutputValue(index, getUnsignedByteValue(readData, USER_OUTPUT_VALUE_OFFSET+index));
        }

        config.setRevLimitRpm(getUnsignedByteValue(readData, REV_LIMIT_OFFSET));
        config.setShiftLightRpm(getUnsignedByteValue(readData, SHIFT_LIGHT_OFFSET));
        
        return config;
    }
    
    public void dataReceived(java.util.List inBuffer) {}
    
 
}
