/*
 * AlertDialog.java
 *
 */

package org.megaJoltLiteJr.presentation;

/**
 *
 * @author  brent
 */
import java.awt.*;

public class AlertDialog extends javax.swing.JDialog {
    /** A return status code - returned if Cancel button has been pressed */
    
    private static final int MESSAGE_PADDING_WIDTH=10;
    private static final int DIALOG_HEIGHT=100;
    
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;
    
    public static final int OK_CANCEL_MODE=1;
    public static final int OK_MODE=0;
    
    
    /** Creates new form AlertDialog */
    public AlertDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }
    
    public static void showMessageDialog(String message){
    
        AlertDialog dlg=new AlertDialog(null, true);
        
        dlg.setText(message);
        dlg.setMode(AlertDialog.OK_MODE);
        dlg.show();
    }
    
    public static int showConfirmationDialog(String message){
        
        AlertDialog dlg=new AlertDialog(null, true);
        
        dlg.setText(message);
        dlg.setMode(AlertDialog.OK_CANCEL_MODE);
        dlg.show();
        return dlg.getReturnStatus();
    }
    
    

    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
        return returnStatus;
    }
    
    public void setMode(int mode){
     
        this.cancelButton.setVisible(OK_CANCEL_MODE==mode);
    }
    
    public void setText(String newText){

        this.dialogLabel.setText(newText);
        FontMetrics fm=this.dialogLabel.getFontMetrics(this.dialogLabel.getFont());
        
        int textWidth=fm.stringWidth(getText()) +(MESSAGE_PADDING_WIDTH*2);
        setSize(textWidth,DIALOG_HEIGHT);        
        
    }
    
    public String getText(){
        return this.dialogLabel.getText();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        dialogLabel = new javax.swing.JLabel();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        okButton.setFont(new java.awt.Font("Dialog", 0, 10));
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setFont(new java.awt.Font("Dialog", 0, 10));
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        dialogLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        dialogLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        getContentPane().add(dialogLabel, java.awt.BorderLayout.CENTER);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-250)/2, (screenSize.height-100)/2, 250, 100);
    }//GEN-END:initComponents
    
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed
    
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose(RET_CANCEL);
    }//GEN-LAST:event_closeDialog
    
    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dispose();
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new AlertDialog(new javax.swing.JFrame(), true).show();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel dialogLabel;
    private javax.swing.JButton okButton;
    // End of variables declaration//GEN-END:variables
    
    private int returnStatus = RET_CANCEL;
}
