/*
 * ErrorDialog.java
 *
 */

package org.megaJoltLiteJr.presentation;

/**
 *
 * @author  brent
 */
import java.io.*;

public class ErrorDialog extends javax.swing.JDialog {
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;
    
    /** Creates new form ErrorDialog */
    public ErrorDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        this.detailedInfoScrollPane.setVisible(false);
    }
    
    private void initDetails(){
     
        this.errorMessageLabel.setText(getMessage());
        Throwable t=getThrowable();
        
        if (null!=t){

            StringWriter w=new StringWriter();
            PrintWriter pw = new PrintWriter(w);
            
            t.printStackTrace(pw);

            this.detailedInfoTextArea.setText(t.getMessage() + "\n\n" + w.toString());
        }
    }
    
    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
        return returnStatus;
    }
    
    public void show(){
        
        initDetails();
        super.show();
    }
    
    public static void showErrorDialog(String message, Throwable t){
     
        ErrorDialog dlg=new ErrorDialog(null,true);

        //change this to use the logging system in the future
        
        dlg.setMessage(message);
        dlg.setThrowable(t);
        dlg.show();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        errorDetailsPanel = new javax.swing.JPanel();
        errorLabel = new javax.swing.JLabel();
        errorMessageLabel = new javax.swing.JLabel();
        moreInfoButton = new javax.swing.JButton();
        detailedInfoScrollPane = new javax.swing.JScrollPane();
        detailedInfoTextArea = new javax.swing.JTextArea();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        okButton.setFont(new java.awt.Font("Dialog", 0, 10));
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        errorDetailsPanel.setLayout(new java.awt.GridBagLayout());

        errorLabel.setText("An Error Occured:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        errorDetailsPanel.add(errorLabel, gridBagConstraints);

        errorMessageLabel.setFont(new java.awt.Font("Dialog", 0, 12));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.weightx = 1.0;
        errorDetailsPanel.add(errorMessageLabel, gridBagConstraints);

        moreInfoButton.setFont(new java.awt.Font("Dialog", 0, 12));
        moreInfoButton.setText("More ->");
        moreInfoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moreInfoButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        errorDetailsPanel.add(moreInfoButton, gridBagConstraints);

        detailedInfoTextArea.setEditable(false);
        detailedInfoScrollPane.setViewportView(detailedInfoTextArea);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        errorDetailsPanel.add(detailedInfoScrollPane, gridBagConstraints);

        getContentPane().add(errorDetailsPanel, java.awt.BorderLayout.CENTER);

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(512, 197));
        setLocation((screenSize.width-512)/2,(screenSize.height-197)/2);
    }//GEN-END:initComponents

    private void moreInfoButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moreInfoButtonActionPerformed
        // Add your handling code here:
        this.detailedInfoScrollPane.setVisible(true);
        this.validate();
    }//GEN-LAST:event_moreInfoButtonActionPerformed
    
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        doClose(RET_OK);
    }//GEN-LAST:event_okButtonActionPerformed
        
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose(RET_OK);
    }//GEN-LAST:event_closeDialog
    
    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dispose();
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new ErrorDialog(new javax.swing.JFrame(), true).show();
    }
    
    /** Getter for property message.
     * @return Value of property message.
     *
     */
    public String getMessage() {
        return this.message;
    }
    
    /** Setter for property message.
     * @param message New value of property message.
     *
     */
    public void setMessage(String message) {
        this.message = message;
    }
    
    /** Getter for property throwable.
     * @return Value of property throwable.
     *
     */
    public Throwable getThrowable() {
        return this.throwable;
    }
    
    /** Setter for property throwable.
     * @param throwable New value of property throwable.
     *
     */
    public void setThrowable(Throwable throwable) {
        this.throwable = throwable;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton okButton;
    private javax.swing.JPanel errorDetailsPanel;
    private javax.swing.JLabel errorMessageLabel;
    private javax.swing.JTextArea detailedInfoTextArea;
    private javax.swing.JButton moreInfoButton;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JScrollPane detailedInfoScrollPane;
    private javax.swing.JLabel errorLabel;
    // End of variables declaration//GEN-END:variables
    
    private int returnStatus = RET_OK;
    
    /** Holds value of property message. */
    private String message;
    
    /** Holds value of property throwable. */
    private Throwable throwable;
    
}
